/*
 * Dan Armendariz
 * danallan@cs.harvard.edu
 *
 * More dictionaries, higher-order functions (HOFs)
 *
 * Run this code in an Xcode Playground by double-clicking its file in Finder.
 */

// Return a pretty string that describes the gradebook
func gradeCount(gradebook: [String:[Int]]) -> String
{
    // check if the gradebook has no values
    if gradebook.isEmpty
    {
        return "The gradebook has no grades."
    }
    
    // gradebook has grades: concatenate all the names of assessments
    let listOfNames = ", ".join(gradebook.keys)
    return "The gradebook has \(gradebook.count) types of assessments: \(listOfNames)"
}

// Return the average of a list of Ints as an optional Double
func average(numbers: [Int]) -> Double?
{
    // return nil if we cannot compute the average
    if numbers.isEmpty
    {
        return nil
    }
    
    // use a higher-order function to sum all values
    let sum = numbers.reduce(0, +)
    
    //equivalent to closure shorthand:
    // let sum = numbers.reduce(0, { $0 + $1 })
    
    return Double(sum) / Double(numbers.count)
}

// Return three values, given an array of Ints: its min value, max, and average
func statistics(numbers: [Int]) ->
    (min: Int, max: Int, avg: Double, passing: [Int], letters: [Character])?
{
    // return nil if there are no values to compute statistics
    if numbers.isEmpty
    {
        return nil
    }
    
    // use the Swift function "min" which returns the lower of two values to find the minimum
    let minimum = numbers.reduce(Int.max, min)
    
    // use the Swift function "max" which returns the higher of two values to find the maximum
    let maximum = numbers.reduce(Int.min, max)

    // filter selects elements of an array that pass a condition
    let passingCount = numbers.filter({ $0 >= 70 })

    // create an array of letter grades
    func gradeToLetter(grade: Int) -> Character
    {
        var letter: Character
        // it's possible to use ranges in a switch statement
        // Note: unlike C, case statements do not "fall through"; no break is needed
        switch grade
        {
            case 90 ... 100:
                letter = "A"
            case 80 ..< 90:
                letter = "B"
            case 70 ..< 80:
                letter = "C"
            case 60 ..< 70:
                letter = "D"
            default:
                letter = "F"
        }
        return letter
    }
    
    // map over every element and transform it to a letter grade
    let letterGrades = numbers.map(gradeToLetter)
    
    return (minimum, maximum, average(numbers)!, passingCount, letterGrades)
}

// Redefine the gradebook as a dictionary with Strings as keys and an array of Ints as values
var gradebook: [String: [Int]] = [:]

// we expect no data at this point
println(gradeCount(gradebook))

// input some data
gradebook["Quiz"] = [70]
gradebook["Psets"] = [100, 85, 90]
gradebook["Final Project"] = []

// sanity check
println(gradeCount(gradebook))

// Playgrounds allow us to see the data structure directly
gradebook

// fetching data must also be safe, we cannot simply do this:
// average(grades["Quiz"])
// because grades["Quizzes"] may not exist. 
// Instead, the Dictionary will return an optional value. Therefore:
if let quizGrades = gradebook["Quizzes"]
{
    if let avg = average(quizGrades)
    {
        println("Average: \(avg)")
    }
    else
    {
        println("No grades in this assessment")
    }
}
else
{
    println("Quizzes doesn't exist in grades D:")
}

// We can also iterate over dictionaries. Let's do that!
for (assessment, grades) in gradebook
{
    if let stats = statistics(grades)
    {
        println("Stats for \(assessment): min of \(stats.min), max of \(stats.max)")
        println("Average: \(stats.avg)")
        println("Number of grades that are passing: \(stats.passing.count)")
        println("Letter grades: \(stats.letters)\n")
    }
    else
    {
        println("Cannot calculate statistics for \(assessment): no grades!")
    }
}
